@@ -1,7 +1,7 @@
 from typing import Any, Dict
 
 from databento.common.parsing import int_to_compression, int_to_schema, int_to_stype
-from dbz_lib import decode_metadata
+from dbz_python import decode_metadata
 
 
 class MetadataDecoder:
@@ -37,6 +37,8 @@ class MetadataDecoder:
             "stype_in": enum_value(int_to_stype),
             "stype_out": enum_value(int_to_stype),
         }
+
         for key, conv_fn in conversion_mapping.items():
             metadata[key] = conv_fn(metadata[key])
+
         return metadata
